import unittest
from unittest.mock import MagicMock
from json import JSONDecodeError
from pygeai.lab.processes.clients import AgenticProcessClient


class TestAgenticProcessClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.lab.processes.test_clients.TestAgenticProcessClient
    """

    def setUp(self):
        self.client = AgenticProcessClient()
        self.client.api_service = MagicMock()
        self.project_id = "test-project-id"
        self.process_id = "test-process-id"
        self.process_name = "test-process-name"
        self.task_id = "test-task-id"
        self.instance_id = "test-instance-id"
        self.thread_id = "test-thread-id"
        self.kb_id = "test-kb-id"
        self.revision = "1"
        self.mock_response = MagicMock()
        self.mock_response.json.return_value = {"status": "success"}
        self.mock_response.text = "success text"

    def test_create_process_success(self):
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_process(
            project_id=self.project_id,
            key="test-key",
            name="Test Process",
            description="Test Description",
            automatic_publish=True
        )

        self.client.api_service.post.assert_called_once()
        self.assertEqual(result, {"status": "success"})

    def test_create_process_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_process(
            project_id=self.project_id,
            key="test-key",
            name="Test Process"
        )

        self.assertEqual(result, "success text")

    def test_update_process_missing_identifier(self):
        with self.assertRaises(ValueError) as context:
            self.client.update_process(project_id=self.project_id)

        self.assertEqual(str(context.exception), "Either process_id or name must be provided.")

    def test_update_process_success_with_id(self):
        self.client.api_service.put.return_value = self.mock_response

        result = self.client.update_process(
            project_id=self.project_id,
            process_id=self.process_id,
            name="Updated Process",
            automatic_publish=True
        )

        self.client.api_service.put.assert_called_once()
        self.assertEqual(result, {"status": "success"})

    def test_get_process_missing_identifier(self):
        with self.assertRaises(ValueError) as context:
            self.client.get_process(project_id=self.project_id)

        self.assertEqual(str(context.exception), "Either process_id or process_name must be provided.")

    def test_get_process_success_with_id(self):
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.get_process(
            project_id=self.project_id,
            process_id=self.process_id,
            revision="0"
        )

        self.client.api_service.get.assert_called_once()
        self.assertEqual(result, {"status": "success"})

    def test_list_processes_success(self):
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.list_processes(
            project_id=self.project_id,
            name="Test Process",
            allow_draft=True
        )

        self.client.api_service.get.assert_called_once()
        self.assertEqual(result, {"status": "success"})

    def test_list_process_instances_missing_process_id(self):
        with self.assertRaises(ValueError) as context:
            self.client.list_process_instances(project_id=self.project_id, process_id="")

        self.assertEqual(str(context.exception), "Process ID must be provided.")

    def test_delete_process_missing_identifier(self):
        with self.assertRaises(ValueError) as context:
            self.client.delete_process(project_id=self.project_id)

        self.assertEqual(str(context.exception), "Either process_id or process_name must be provided.")

    def test_publish_process_revision_missing_revision(self):
        with self.assertRaises(ValueError) as context:
            self.client.publish_process_revision(
                project_id=self.project_id,
                process_id=self.process_id
            )

        self.assertEqual(str(context.exception), "Revision must be provided.")

    def test_create_task_success(self):
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_task(
            project_id=self.project_id,
            name="Test Task",
            description="Task Description"
        )

        self.client.api_service.post.assert_called_once()
        self.assertEqual(result, {"status": "success"})

    def test_get_task_missing_identifier(self):
        with self.assertRaises(ValueError) as context:
            self.client.get_task(project_id=self.project_id, task_id="", task_name="")

        self.assertEqual(str(context.exception), "Either task_id or task_name must be provided.")

    def test_start_instance_success(self):
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.start_instance(
            project_id=self.project_id,
            process_name=self.process_name,
            subject="Test Subject"
        )

        self.client.api_service.post.assert_called_once()
        self.assertEqual(result, {"status": "success"})

    def test_abort_instance_missing_id(self):
        with self.assertRaises(ValueError) as context:
            self.client.abort_instance(project_id=self.project_id, instance_id="")

        self.assertEqual(str(context.exception), "Instance ID must be provided.")

    def test_send_user_signal_missing_signal_name(self):
        with self.assertRaises(ValueError) as context:
            self.client.send_user_signal(
                project_id=self.project_id,
                instance_id=self.instance_id,
                signal_name=""
            )

        self.assertEqual(str(context.exception), "Signal name must be provided.")

    def test_create_kb_success(self):
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_kb(
            project_id=self.project_id,
            name="Test KB",
            artifacts=["artifact1"]
        )

        self.client.api_service.post.assert_called_once()
        self.assertEqual(result, {"status": "success"})

    def test_list_jobs_success(self):
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.list_jobs(
            project_id=self.project_id,
            topic="test-topic"
        )

        self.client.api_service.get.assert_called_once()
        self.assertEqual(result, {"status": "success"})




import unittest
from unittest.mock import MagicMock
from json import JSONDecodeError
from pygeai.lab.tools.clients import ToolClient, VALID_SCOPES, VALID_ACCESS_SCOPES, VALID_REPORT_EVENTS


class TestToolClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.lab.tools.test_clients.TestToolClient
    """

    def setUp(self):
        self.tool_client = ToolClient()
        self.mock_api_service = MagicMock()
        self.tool_client.api_service = self.mock_api_service
        self.project_id = "project-123"

    def test_create_tool_success(self):
        name = "TestTool"
        description = "A test tool"
        scope = "builtin"
        access_scope = "private"
        public_name = "test-tool"
        icon = "http://example.com/icon.png"
        open_api = "http://example.com/api"
        open_api_json = {"info": {"title": "Test API"}}
        report_events = "All"
        parameters = [{"key": "param1", "dataType": "String", "description": "Param 1", "isRequired": True}]
        automatic_publish = True
        expected_response = {"id": "tool-123", "name": name}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.post.return_value = mock_response

        result = self.tool_client.create_tool(
            project_id=self.project_id,
            name=name,
            description=description,
            scope=scope,
            access_scope=access_scope,
            public_name=public_name,
            icon=icon,
            open_api=open_api,
            open_api_json=open_api_json,
            report_events=report_events,
            parameters=parameters,
            automatic_publish=automatic_publish
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.post.assert_called_once()
        call_args = self.mock_api_service.post.call_args
        data = call_args[1]['data']['tool']
        self.assertEqual(data['name'], name)
        self.assertEqual(data['description'], description)
        self.assertEqual(data['scope'], scope)
        self.assertEqual(data['accessScope'], access_scope)
        self.assertEqual(data['publicName'], public_name)
        self.assertEqual(data['icon'], icon)
        self.assertEqual(data['openApi'], open_api)
        self.assertIsInstance(data['openApiJson'], str)
        self.assertEqual(data['reportEvents'], report_events)
        self.assertEqual(data['parameters'], parameters)
        self.assertIn("automaticPublish=true", call_args[1]['endpoint'])
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_create_tool_invalid_scope(self):
        with self.assertRaises(ValueError) as context:
            self.tool_client.create_tool(
                project_id=self.project_id,
                name="TestTool",
                scope="invalid_scope"
            )
        self.assertEqual(str(context.exception), f"Scope must be one of {', '.join(VALID_SCOPES)}.")

    def test_create_tool_invalid_access_scope(self):
        with self.assertRaises(ValueError) as context:
            self.tool_client.create_tool(
                project_id=self.project_id,
                name="TestTool",
                access_scope="invalid_access"
            )
        self.assertEqual(str(context.exception), f"Access scope must be one of {', '.join(VALID_ACCESS_SCOPES)}.")

    def test_create_tool_invalid_report_events(self):
        with self.assertRaises(ValueError) as context:
            self.tool_client.create_tool(
                project_id=self.project_id,
                name="TestTool",
                report_events="invalid_event"
            )
        self.assertEqual(str(context.exception), f"Report events must be one of {', '.join(VALID_REPORT_EVENTS)}.")

    def test_list_tools_success(self):
        expected_response = {"tools": [{"id": "tool-1", "name": "Tool1"}]}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.get.return_value = mock_response

        result = self.tool_client.list_tools(
            project_id=self.project_id,
            id="tool-1",
            count="50",
            access_scope="public",
            allow_drafts=True,
            scope="api",
            allow_external=True
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.get.assert_called_once()
        call_args = self.mock_api_service.get.call_args
        params = call_args[1]['params']
        self.assertEqual(params['id'], "tool-1")
        self.assertEqual(params['count'], "50")
        self.assertEqual(params['accessScope'], "public")
        self.assertTrue(params['allowDrafts'])
        self.assertEqual(params['scope'], "api")
        self.assertTrue(params['allowExternal'])
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_list_tools_invalid_scope(self):
        with self.assertRaises(ValueError) as context:
            self.tool_client.list_tools(
                project_id=self.project_id,
                scope="invalid_scope"
            )
        self.assertEqual(str(context.exception), f"Scope must be one of {', '.join(VALID_SCOPES)}.")

    def test_get_tool_success(self):
        tool_id = "tool-123"
        expected_response = {"id": tool_id, "name": "TestTool"}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.get.return_value = mock_response

        result = self.tool_client.get_tool(
            project_id=self.project_id,
            tool_id=tool_id,
            revision="1",
            version=0,
            allow_drafts=True
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.get.assert_called_once()
        call_args = self.mock_api_service.get.call_args
        params = call_args[1]['params']
        self.assertEqual(params['revision'], "1")
        self.assertEqual(params['version'], 0)
        self.assertTrue(params['allowDrafts'])
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_delete_tool_success_with_id(self):
        tool_id = "tool-123"
        expected_response = {"status": "deleted"}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.delete.return_value = mock_response

        result = self.tool_client.delete_tool(
            project_id=self.project_id,
            tool_id=tool_id
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.delete.assert_called_once()
        headers = self.mock_api_service.delete.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_delete_tool_success_with_name(self):
        tool_name = "TestTool"
        expected_response = {"status": "deleted"}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.delete.return_value = mock_response

        result = self.tool_client.delete_tool(
            project_id=self.project_id,
            tool_name=tool_name
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.delete.assert_called_once()

    def test_delete_tool_invalid_input(self):
        with self.assertRaises(ValueError) as context:
            self.tool_client.delete_tool(project_id=self.project_id)
        self.assertEqual(str(context.exception), "Either tool_id or tool_name must be provided.")

    def test_update_tool_success(self):
        tool_id = "tool-123"
        name = "UpdatedTool"
        description = "Updated description"
        scope = "external"
        access_scope = "public"
        public_name = "updated-tool"
        icon = "http://example.com/new-icon.png"
        open_api = "http://example.com/new-api"
        open_api_json = {"info": {"title": "Updated API"}}
        report_events = "Start"
        parameters = [{"key": "param1", "dataType": "String", "description": "Param 1", "isRequired": True}]
        automatic_publish = True
        upsert = False
        expected_response = {"id": tool_id, "name": name}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.put.return_value = mock_response

        result = self.tool_client.update_tool(
            project_id=self.project_id,
            tool_id=tool_id,
            name=name,
            description=description,
            scope=scope,
            access_scope=access_scope,
            public_name=public_name,
            icon=icon,
            open_api=open_api,
            open_api_json=open_api_json,
            report_events=report_events,
            parameters=parameters,
            automatic_publish=automatic_publish,
            upsert=upsert
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.put.assert_called_once()
        call_args = self.mock_api_service.put.call_args
        data = call_args[1]['data']['tool']
        self.assertEqual(data['name'], name)
        self.assertEqual(data['description'], description)
        self.assertEqual(data['scope'], scope)
        self.assertEqual(data['accessScope'], access_scope)
        self.assertEqual(data['publicName'], public_name)
        self.assertEqual(data['icon'], icon)
        self.assertEqual(data['openApi'], open_api)
        self.assertIsInstance(data['openApiJson'], str)
        self.assertEqual(data['reportEvents'], report_events)
        self.assertEqual(data['parameters'], parameters)
        self.assertIn("automaticPublish=true", call_args[1]['endpoint'])
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_update_tool_invalid_scope(self):
        with self.assertRaises(ValueError) as context:
            self.tool_client.update_tool(
                project_id=self.project_id,
                tool_id="tool-123",
                scope="invalid_scope"
            )
        self.assertEqual(str(context.exception), f"Scope must be one of {', '.join(VALID_SCOPES)}.")

    def test_publish_tool_revision_success(self):
        tool_id = "tool-123"
        revision = "2"
        expected_response = {"status": "published"}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.post.return_value = mock_response

        result = self.tool_client.publish_tool_revision(
            project_id=self.project_id,
            tool_id=tool_id,
            revision=revision
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.post.assert_called_once()
        call_args = self.mock_api_service.post.call_args
        data = call_args[1]['data']
        self.assertEqual(data['revision'], revision)
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_get_parameter_success_with_id(self):
        tool_id = "tool-123"
        expected_response = {"parameters": [{"key": "param1"}]}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.get.return_value = mock_response

        result = self.tool_client.get_parameter(
            project_id=self.project_id,
            tool_id=tool_id,
            revision="1",
            version=0,
            allow_drafts=True
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.get.assert_called_once()
        call_args = self.mock_api_service.get.call_args
        params = call_args[1]['params']
        self.assertEqual(params['revision'], "1")
        self.assertEqual(params['version'], 0)
        self.assertTrue(params['allowDrafts'])
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_get_parameter_success_with_public_name(self):
        tool_public_name = "test-tool"
        expected_response = {"parameters": [{"key": "param1"}]}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.get.return_value = mock_response

        result = self.tool_client.get_parameter(
            project_id=self.project_id,
            tool_public_name=tool_public_name,
            revision="1",
            version=0,
            allow_drafts=True
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.get.assert_called_once()

    def test_get_parameter_invalid_input(self):
        with self.assertRaises(ValueError) as context:
            self.tool_client.get_parameter(project_id=self.project_id)
        self.assertEqual(str(context.exception), "Either tool_id or tool_public_name must be provided.")

    def test_set_parameter_success_with_id(self):
        tool_id = "tool-123"
        parameters = [{"key": "param1", "dataType": "String", "description": "Param 1", "isRequired": True}]
        expected_response = {"status": "updated"}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.post.return_value = mock_response

        result = self.tool_client.set_parameter(
            project_id=self.project_id,
            tool_id=tool_id,
            parameters=parameters
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.post.assert_called_once()
        call_args = self.mock_api_service.post.call_args
        data = call_args[1]['data']['parameterDefinition']
        self.assertEqual(data['parameters'], parameters)
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_set_parameter_success_with_public_name(self):
        tool_public_name = "test-tool"
        parameters = [{"key": "param1", "dataType": "String", "description": "Param 1", "isRequired": True}]
        expected_response = {"status": "updated"}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.post.return_value = mock_response

        result = self.tool_client.set_parameter(
            project_id=self.project_id,
            tool_public_name=tool_public_name,
            parameters=parameters
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.post.assert_called_once()

    def test_set_parameter_invalid_input(self):
        with self.assertRaises(ValueError) as context:
            self.tool_client.set_parameter(project_id=self.project_id, parameters=[{"key": "param1"}])
        self.assertEqual(str(context.exception), "Either tool_id or tool_public_name must be provided.")

        with self.assertRaises(ValueError) as context:
            self.tool_client.set_parameter(project_id=self.project_id, tool_id="tool-123", parameters=[])
        self.assertEqual(str(context.exception), "Parameters list must be provided and non-empty.")

    def test_create_tool_json_decode_error(self):
        name = "TestTool"
        mock_response = MagicMock()
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "error text"
        self.mock_api_service.post.return_value = mock_response

        result = self.tool_client.create_tool(
            project_id=self.project_id,
            name=name
        )

        self.assertEqual(result, "error text")
        self.mock_api_service.post.assert_called_once()

    def test_list_tools_json_decode_error(self):
        mock_response = MagicMock()
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "error text"
        self.mock_api_service.get.return_value = mock_response

        result = self.tool_client.list_tools(
            project_id=self.project_id,
            scope="api"
        )

        self.assertEqual(result, "error text")
        self.mock_api_service.get.assert_called_once()

    def test_get_tool_json_decode_error(self):
        tool_id = "tool-123"
        mock_response = MagicMock()
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "error text"
        self.mock_api_service.get.return_value = mock_response

        result = self.tool_client.get_tool(
            project_id=self.project_id,
            tool_id=tool_id
        )

        self.assertEqual(result, "error text")
        self.mock_api_service.get.assert_called_once()

    def test_delete_tool_json_decode_error(self):
        tool_id = "tool-123"
        mock_response = MagicMock()
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "error text"
        self.mock_api_service.delete.return_value = mock_response

        result = self.tool_client.delete_tool(
            project_id=self.project_id,
            tool_id=tool_id
        )

        self.assertEqual(result, "error text")
        self.mock_api_service.delete.assert_called_once()

    def test_update_tool_json_decode_error(self):
        tool_id = "tool-123"
        mock_response = MagicMock()
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "error text"
        self.mock_api_service.put.return_value = mock_response

        result = self.tool_client.update_tool(
            project_id=self.project_id,
            tool_id=tool_id,
            name="UpdatedTool"
        )

        self.assertEqual(result, "error text")
        self.mock_api_service.put.assert_called_once()

    def test_publish_tool_revision_json_decode_error(self):
        tool_id = "tool-123"
        revision = "2"
        mock_response = MagicMock()
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "error text"
        self.mock_api_service.post.return_value = mock_response

        result = self.tool_client.publish_tool_revision(
            project_id=self.project_id,
            tool_id=tool_id,
            revision=revision
        )

        self.assertEqual(result, "error text")
        self.mock_api_service.post.assert_called_once()

    def test_get_parameter_json_decode_error(self):
        tool_id = "tool-123"
        mock_response = MagicMock()
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "error text"
        self.mock_api_service.get.return_value = mock_response

        result = self.tool_client.get_parameter(
            project_id=self.project_id,
            tool_id=tool_id
        )

        self.assertEqual(result, "error text")
        self.mock_api_service.get.assert_called_once()

    def test_set_parameter_json_decode_error(self):
        tool_id = "tool-123"
        parameters = [{"key": "param1", "dataType": "String", "description": "Param 1", "isRequired": True}]
        mock_response = MagicMock()
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "error text"
        self.mock_api_service.post.return_value = mock_response

        result = self.tool_client.set_parameter(
            project_id=self.project_id,
            tool_id=tool_id,
            parameters=parameters
        )

        self.assertEqual(result, "error text")
        self.mock_api_service.post.assert_called_once()

    def test_update_tool_with_upsert(self):
        tool_id = "tool-123"
        name = "UpsertedTool"
        expected_response = {"id": tool_id, "name": name}
        mock_response = MagicMock()
        mock_response.json.return_value = expected_response
        self.mock_api_service.put.return_value = mock_response

        result = self.tool_client.update_tool(
            project_id=self.project_id,
            tool_id=tool_id,
            name=name,
            upsert=True
        )

        self.assertEqual(result, expected_response)
        self.mock_api_service.put.assert_called_once()
        call_args = self.mock_api_service.put.call_args
        endpoint = call_args[1]['endpoint']
        self.assertIn("upsert", endpoint)




from pygeai.core.models import Organization, Project
from pygeai.core.files.managers import FileManager
from pygeai.core.files.models import File

organization = Organization(id="4aa15b61-d3c7-4a5c-99b8-052d18a04ff2")
project = Project(id="1956c032-3c66-4435-acb8-6a06e52f819f")
file = File(id="7fe2393d-8f86-4020-a51f-bc14ab957e1e")

file_manager = FileManager(organization_id=organization.id, project_id=project.id)

response = file_manager.delete_file(file_id=file.id)
print(response)



import unittest
from unittest.mock import MagicMock
from json import JSONDecodeError
from pygeai.lab.agents.clients import AgentClient


class TestAgentClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.lab.agents.test_clients.TestAgentClient
    """

    def setUp(self):
        self.client = AgentClient()
        self.client.api_service = MagicMock()
        self.project_id = "test-project-id"
        self.agent_id = "test-agent-id"
        self.mock_response = MagicMock()
        self.mock_response.json.return_value = {"status": "success"}
        self.mock_response.text = "success text"

    def test_list_agents_success(self):
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.list_agents(
            project_id=self.project_id,
            status="active",
            start=0,
            count=10,
            access_scope="private",
            allow_drafts=True,
            allow_external=True
        )

        self.client.api_service.get.assert_called_once()
        params = self.client.api_service.get.call_args[1]['params']
        self.assertEqual(params['status'], "active")
        self.assertEqual(params['start'], 0)
        self.assertEqual(params['count'], 10)
        self.assertEqual(params['accessScope'], "private")
        self.assertTrue(params['allowDrafts'])
        self.assertTrue(params['allowExternal'])
        headers = self.client.api_service.get.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)
        self.assertEqual(result, {"status": "success"})

    def test_list_agents_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.list_agents(
            project_id=self.project_id
        )

        self.assertEqual(result, "success text")
        self.client.api_service.get.assert_called_once()

    def test_create_agent_success(self):
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_agent(
            project_id=self.project_id,
            name="Test Agent",
            access_scope="private",
            public_name="test-agent",
            job_description="Agent Role",
            avatar_image="http://example.com/avatar.png",
            description="Agent Description",
            agent_data_prompt={"instructions": "Do this task"},
            agent_data_llm_config={"maxTokens": 100, "timeout": 30},
            agent_data_models=[{"name": "gpt-4o"}],
            agent_data_resource_pools=[{"name": "pool1", "tools": [{"name": "tool1"}]}],
            automatic_publish=True
        )

        self.client.api_service.post.assert_called_once()
        data = self.client.api_service.post.call_args[1]['data']['agentDefinition']
        self.assertEqual(data['name'], "Test Agent")
        self.assertEqual(data['accessScope'], "private")
        self.assertEqual(data['publicName'], "test-agent")
        self.assertEqual(data['jobDescription'], "Agent Role")
        self.assertEqual(data['avatarImage'], "http://example.com/avatar.png")
        self.assertEqual(data['description'], "Agent Description")
        self.assertIn("prompt", data['agentData'])
        self.assertIn("llmConfig", data['agentData'])
        self.assertIn("models", data['agentData'])
        self.assertIn("resourcePools", data['agentData'])
        endpoint = self.client.api_service.post.call_args[1]['endpoint']
        self.assertIn("automaticPublish=true", endpoint)
        headers = self.client.api_service.post.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)
        self.assertEqual(result, {"status": "success"})

    def test_create_agent_without_resource_pools(self):
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_agent(
            project_id=self.project_id,
            name="Test Agent",
            access_scope="private",
            public_name="test-agent",
            job_description="Agent Role",
            avatar_image="http://example.com/avatar.png",
            description="Agent Description",
            agent_data_prompt={"instructions": "Do this task"},
            agent_data_llm_config={"maxTokens": 100},
            agent_data_models=[{"name": "gpt-4o"}],
            automatic_publish=False
        )

        self.client.api_service.post.assert_called_once()
        data = self.client.api_service.post.call_args[1]['data']['agentDefinition']
        self.assertNotIn("resourcePools", data['agentData'])
        self.assertEqual(result, {"status": "success"})

    def test_create_agent_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_agent(
            project_id=self.project_id,
            name="Test Agent",
            access_scope="private",
            public_name="test-agent",
            job_description="Agent Role",
            avatar_image="http://example.com/avatar.png",
            description="Agent Description",
            agent_data_prompt={"instructions": "Do this task"},
            agent_data_llm_config={"maxTokens": 100},
            agent_data_models=[{"name": "gpt-4o"}]
        )

        self.assertEqual(result, "success text")
        self.client.api_service.post.assert_called_once()

    def test_get_agent_success(self):
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.get_agent(
            project_id=self.project_id,
            agent_id=self.agent_id,
            revision="1",
            version=2,
            allow_drafts=False
        )

        self.client.api_service.get.assert_called_once()
        params = self.client.api_service.get.call_args[1]['params']
        self.assertEqual(params['revision'], "1")
        self.assertEqual(params['version'], 2)
        self.assertFalse(params['allowDrafts'])
        headers = self.client.api_service.get.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)
        self.assertEqual(result, {"status": "success"})

    def test_get_agent_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.get_agent(
            project_id=self.project_id,
            agent_id=self.agent_id
        )

        self.assertEqual(result, "success text")
        self.client.api_service.get.assert_called_once()

    def test_create_sharing_link_success(self):
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.create_sharing_link(
            project_id=self.project_id,
            agent_id=self.agent_id
        )

        self.client.api_service.get.assert_called_once()
        headers = self.client.api_service.get.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)
        self.assertEqual(result, {"status": "success"})

    def test_create_sharing_link_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.create_sharing_link(
            project_id=self.project_id,
            agent_id=self.agent_id
        )

        self.assertEqual(result, "success text")
        self.client.api_service.get.assert_called_once()

    def test_publish_agent_revision_success(self):
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.publish_agent_revision(
            project_id=self.project_id,
            agent_id=self.agent_id,
            revision="2"
        )

        self.client.api_service.post.assert_called_once()
        data = self.client.api_service.post.call_args[1]['data']
        self.assertEqual(data['revision'], "2")
        headers = self.client.api_service.post.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)
        self.assertEqual(result, {"status": "success"})

    def test_publish_agent_revision_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.publish_agent_revision(
            project_id=self.project_id,
            agent_id=self.agent_id,
            revision="2"
        )

        self.assertEqual(result, "success text")
        self.client.api_service.post.assert_called_once()

    def test_delete_agent_success(self):
        self.client.api_service.delete.return_value = self.mock_response

        result = self.client.delete_agent(
            project_id=self.project_id,
            agent_id=self.agent_id
        )

        self.client.api_service.delete.assert_called_once()
        headers = self.client.api_service.delete.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)
        self.assertEqual(result, {"status": "success"})

    def test_delete_agent_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.client.api_service.delete.return_value = self.mock_response

        result = self.client.delete_agent(
            project_id=self.project_id,
            agent_id=self.agent_id
        )

        self.assertEqual(result, "success text")
        self.client.api_service.delete.assert_called_once()

    def test_update_agent_success(self):
        self.client.api_service.put.return_value = self.mock_response

        result = self.client.update_agent(
            project_id=self.project_id,
            agent_id=self.agent_id,
            name="Updated Agent",
            access_scope="public",
            public_name="updated-agent",
            job_description="Updated Role",
            avatar_image="http://example.com/new-avatar.png",
            description="Updated Description",
            agent_data_prompt={"instructions": "Do this updated task"},
            agent_data_llm_config={"maxTokens": 200},
            agent_data_models=[{"name": "gpt-4o-updated"}],
            agent_data_resource_pools=[{"name": "pool2", "tools": [{"name": "tool2"}]}],
            automatic_publish=True,
            upsert=False
        )

        self.client.api_service.put.assert_called_once()
        data = self.client.api_service.put.call_args[1]['data']['agentDefinition']
        self.assertEqual(data['name'], "Updated Agent")
        self.assertEqual(data['accessScope'], "public")
        self.assertEqual(data['publicName'], "updated-agent")
        self.assertEqual(data['jobDescription'], "Updated Role")
        self.assertEqual(data['avatarImage'], "http://example.com/new-avatar.png")
        self.assertEqual(data['description'], "Updated Description")
        self.assertIn("prompt", data['agentData'])
        self.assertIn("llmConfig", data['agentData'])
        self.assertIn("models", data['agentData'])
        self.assertIn("resourcePools", data['agentData'])
        endpoint = self.client.api_service.put.call_args[1]['endpoint']
        self.assertIn("automaticPublish=true", endpoint)
        headers = self.client.api_service.put.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)
        self.assertEqual(result, {"status": "success"})

    def test_update_agent_with_upsert(self):
        self.client.api_service.put.return_value = self.mock_response

        result = self.client.update_agent(
            project_id=self.project_id,
            agent_id=self.agent_id,
            name="Upserted Agent",
            access_scope="private",
            public_name="upserted-agent",
            job_description="Upserted Role",
            avatar_image="http://example.com/upsert-avatar.png",
            description="Upserted Description",
            agent_data_prompt={"instructions": "Do this upserted task"},
            agent_data_llm_config={"maxTokens": 150},
            agent_data_models=[{"name": "gpt-4o-upserted"}],
            automatic_publish=False,
            upsert=True
        )

        self.client.api_service.put.assert_called_once()
        endpoint = self.client.api_service.put.call_args[1]['endpoint']
        self.assertIn("upsert", endpoint)
        self.assertEqual(result, {"status": "success"})

    def test_update_agent_without_resource_pools(self):
        self.client.api_service.put.return_value = self.mock_response

        result = self.client.update_agent(
            project_id=self.project_id,
            agent_id=self.agent_id,
            name="Updated Agent No Pools",
            access_scope="private",
            public_name="updated-agent-no-pools",
            job_description="Updated Role",
            avatar_image="http://example.com/avatar.png",
            description="Updated Description",
            agent_data_prompt={"instructions": "Do this task"},
            agent_data_llm_config={"maxTokens": 100},
            agent_data_models=[{"name": "gpt-4o"}],
            automatic_publish=False,
            upsert=False
        )

        self.client.api_service.put.assert_called_once()
        data = self.client.api_service.put.call_args[1]['data']['agentDefinition']
        self.assertNotIn("resourcePools", data['agentData'])
        self.assertEqual(result, {"status": "success"})

    def test_update_agent_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.client.api_service.put.return_value = self.mock_response

        result = self.client.update_agent(
            project_id=self.project_id,
            agent_id=self.agent_id,
            name="Updated Agent",
            access_scope="private",
            public_name="updated-agent",
            job_description="Updated Role",
            avatar_image="http://example.com/avatar.png",
            description="Updated Description",
            agent_data_prompt={"instructions": "Do this task"},
            agent_data_llm_config={"maxTokens": 100},
            agent_data_models=[{"name": "gpt-4o"}]
        )

        self.assertEqual(result, "success text")
        self.client.api_service.put.assert_called_once()

